@@ -24,6 +24,10 @@ const PARALLELISM: usize = 20;
 /// The size of a block. This is a Dropbox constant, not adjustable.
 const BLOCK_SIZE: usize = 4 * 1024 * 1024;
 
+/// We can upload an integer multiple of BLOCK_SIZE in a single request. This reduces the number of
+/// requests needed to do the upload and can help avoid running into rate limits.
+const BLOCKS_PER_REQUEST: usize = 2;
+
 macro_rules! fatal {
     ($($arg:tt)*) => {
         eprintln!($($arg)*);
@@ -48,7 +52,7 @@ struct Args {
     resume: Option<Resume>,
 }
 
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 struct Resume {
     start_offset: u64,
     session_id: String,
@@ -240,7 +244,7 @@ impl UploadSession {
     /// Mark a block as uploaded.
     pub fn mark_block_uploaded(&self, block_offset: u64, block_len: u64) {
         let mut completion = self.completion.lock().unwrap();
-        completion.complete_block(block_offset, block_len);
+        completion.complete_block(self.start_offset + block_offset, block_len);
     }
 
     /// Return the offset up to which the file is completely uploaded. It can be resumed from this
@@ -312,30 +316,41 @@ fn upload_file(
 
     let (source_mtime, source_len) = get_file_mtime_and_size(&source_file)?;
 
-    let session = Arc::new(if let Some(resume) = resume {
+    let session = Arc::new(if let Some(ref resume) = resume {
         source_file.seek(SeekFrom::Start(resume.start_offset))
             .map_err(|e| format!("Seek error: {}", e))?;
-        UploadSession::resume(resume, source_len)
+        UploadSession::resume(resume.clone(), source_len)
     } else {
         UploadSession::new(client.as_ref(), source_len)?
     });
 
     eprintln!("upload session ID is {}", session.session_id);
 
+    // Initially set to the end of the file and an empty block; if the file is an exact multiple of
+    // BLOCK_SIZE, we'll need to upload an empty buffer when closing the session.
+    let last_block = Arc::new(Mutex::new((source_len, vec![])));
+
     let start_time = Instant::now();
     let upload_result = {
         let client = client.clone();
         let session = session.clone();
+        let last_block = last_block.clone();
+        let resume = resume.clone();
         parallel_reader::read_stream_and_process_chunks_in_parallel(
             &mut source_file,
-            BLOCK_SIZE,
+            BLOCK_SIZE * BLOCKS_PER_REQUEST,
             PARALLELISM,
             Arc::new(move |block_offset, data: &[u8]| -> Result<(), String> {
-                let mut append_arg = session.append_arg(block_offset);
-                if data.len() != BLOCK_SIZE {
+                let append_arg = session.append_arg(block_offset);
+                if data.len() != BLOCK_SIZE * BLOCKS_PER_REQUEST {
                     // This must be the last block. Only the last one is allowed to be not 4 MiB
-                    // exactly, so let's close the session.
-                    append_arg.close = true;
+                    // exactly. Save the block and offset so it can be uploaded after all the
+                    // parallel uploads are done. This is because once the session is closed, we
+                    // can't resume it.
+                    let mut last_block = last_block.lock().unwrap();
+                    last_block.0 = block_offset + session.start_offset;
+                    last_block.1 = data.to_vec();
+                    return Ok(());
                 }
                 let result = upload_block_with_retry(
                     client.as_ref(),
@@ -343,6 +358,7 @@ fn upload_file(
                     data,
                     start_time,
                     session.as_ref(),
+                    resume.as_ref(),
                 );
                 if result.is_ok() {
                     session.mark_block_uploaded(block_offset, data.len() as u64);
@@ -356,6 +372,19 @@ fn upload_file(
             e, session.session_id, session.complete_up_to()));
     }
 
+    let (last_block_offset, last_block_data) = unwrap_arcmutex(last_block);
+    eprintln!("closing session at {} with {}-byte block",
+        last_block_offset, last_block_data.len());
+    let mut arg = session.append_arg(last_block_offset);
+    arg.close = true;
+    if let Err(e) = upload_block_with_retry(
+        client.as_ref(), &arg, &last_block_data, start_time, session.as_ref(), resume.as_ref())
+    {
+        eprintln!("failed to close session: {}", e);
+        // But don't error out; try committing anyway. It could be we're resuming a file where we
+        // already closed it out but failed to commit.
+    }
+
     eprintln!("committing...");
     let finish = session.commit_arg(dest_path, source_mtime);
 
@@ -388,13 +417,18 @@ fn upload_block_with_retry(
     buf: &[u8],
     start_time: Instant,
     session: &UploadSession,
+    resume: Option<&Resume>,
 ) -> Result<(), String> {
     let block_start_time = Instant::now();
     let mut errors = 0;
     loop {
         match files::upload_session_append_v2(client, arg, buf) {
-            Ok(Ok(())) => {
-                break;
+            Ok(Ok(())) => { break; }
+            Err(dropbox_sdk::Error::RateLimited { reason, retry_after_seconds }) => {
+                eprintln!("rate-limited ({}), waiting {} seconds", reason, retry_after_seconds);
+                if retry_after_seconds > 0 {
+                    sleep(Duration::from_secs(u64::from(retry_after_seconds)));
+                }
             }
             error => {
                 errors += 1;
@@ -415,7 +449,8 @@ fn upload_block_with_retry(
     let block_bytes = buf.len() as u64;
     let bytes_sofar = session.bytes_transferred.fetch_add(block_bytes, SeqCst) + block_bytes;
 
-    let percent = bytes_sofar as f64 / session.file_size as f64 * 100.;
+    let percent = (resume.map(|r| r.start_offset).unwrap_or(0) + bytes_sofar) as f64
+        / session.file_size as f64 * 100.;
 
     // This assumes that we have `PARALLELISM` uploads going at the same time and at roughly the
     // same upload speed:
@@ -469,6 +504,13 @@ fn iso8601(t: SystemTime) -> String {
         .format("%Y-%m-%dT%H:%M:%SZ").to_string()
 }
 
+fn unwrap_arcmutex<T: std::fmt::Debug>(x: Arc<Mutex<T>>) -> T {
+    Arc::try_unwrap(x)
+        .expect("failed to unwrap Arc")
+        .into_inner()
+        .expect("failed to unwrap Mutex")
+}
+
 fn main() {
     env_logger::init();
 
